package xyz.lychee.lagfixer.nms.v1_16_R3;

import com.mojang.authlib.GameProfile;
import com.mojang.authlib.properties.Property;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.UUID;
import net.minecraft.server.v1_16_R3.EntityCreature;
import net.minecraft.server.v1_16_R3.EntityPlayer;
import net.minecraft.server.v1_16_R3.GameRules;
import org.apache.commons.lang.RandomStringUtils;
import org.bukkit.Bukkit;
import org.bukkit.Chunk;
import org.bukkit.GameRule;
import org.bukkit.Material;
import org.bukkit.NamespacedKey;
import org.bukkit.World;
import org.bukkit.craftbukkit.v1_16_R3.CraftChunk;
import org.bukkit.craftbukkit.v1_16_R3.CraftServer;
import org.bukkit.craftbukkit.v1_16_R3.CraftWorld;
import org.bukkit.craftbukkit.v1_16_R3.entity.CraftCreature;
import org.bukkit.entity.Entity;
import org.bukkit.entity.Item;
import org.bukkit.entity.LivingEntity;
import org.bukkit.entity.Projectile;
import org.bukkit.event.EventHandler;
import org.bukkit.event.world.ChunkLoadEvent;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;
import org.bukkit.persistence.PersistentDataType;
import org.bukkit.plugin.Plugin;
import xyz.lychee.lagfixer.managers.ModuleManager;
import xyz.lychee.lagfixer.managers.SupportManager;
import xyz.lychee.lagfixer.modules.CustomAiModule;
import xyz.lychee.lagfixer.modules.CustomVehiclesModule;
import xyz.lychee.lagfixer.objects.AbstractSupportNms;

public class SupportNms extends AbstractSupportNms {
   private final int viewDistance = Bukkit.getServer().getViewDistance();
   private final GameRules.GameRuleKey<?> randomTickSpeed;

   public SupportNms(Plugin plugin) {
      super(plugin);
      this.randomTickSpeed = (GameRules.GameRuleKey)CraftWorld.getGameRulesNMS().get(GameRule.RANDOM_TICK_SPEED.getName());
   }

   public int getViewDistance() {
      return ((CraftServer)Bukkit.getServer()).getHandle().getViewDistance();
   }

   public int getViewSimulation() {
      return 0;
   }

   public ItemStack setNBTValue(ItemStack is, String key, String value) {
      ItemMeta meta = is.getItemMeta();
      if (meta != null) {
         meta.getPersistentDataContainer().set(new NamespacedKey(this.getPlugin(), key), PersistentDataType.STRING, value);
         is.setItemMeta(meta);
      }

      return is;
   }

   public boolean hasNBTValue(ItemStack is, String key) {
      ItemMeta meta = is.getItemMeta();
      return meta != null ? meta.getPersistentDataContainer().has(new NamespacedKey(this.getPlugin(), key), PersistentDataType.STRING) : false;
   }

   public String getNBTValue(ItemStack is, String key) {
      ItemMeta meta = is.getItemMeta();
      return meta != null ? (String)meta.getPersistentDataContainer().get(new NamespacedKey(this.getPlugin(), key), PersistentDataType.STRING) : null;
   }

   public ItemStack createSkull(String base64) {
      ItemStack is = new ItemStack(Material.PLAYER_HEAD);
      ItemMeta meta = is.getItemMeta();
      if (meta == null) {
         return is;
      } else {
         GameProfile profile = new GameProfile(UUID.randomUUID(), RandomStringUtils.randomAlphabetic(8));
         profile.getProperties().put("textures", new Property("textures", base64));

         try {
            Method mtd = meta.getClass().getDeclaredMethod("setProfile", GameProfile.class);
            mtd.setAccessible(true);
            mtd.invoke(meta, profile);
         } catch (NoSuchMethodException | InvocationTargetException | IllegalAccessException var6) {
            return is;
         }

         is.setItemMeta(meta);
         return is;
      }
   }

   public void setDefaultDistance() {
      this.setDistance(this.viewDistance, 0);
   }

   public void setRandomTickSpeed(World w, boolean bl) {
      GameRules.GameRuleValue<?> handle = ((CraftWorld)w).getHandle().getGameRules().get(this.randomTickSpeed);
      handle.setValue(bl ? "3" : "0");
   }

   public int getTileEntitiesCount(Chunk c) {
      return c.isLoaded() ? ((CraftChunk)c).getHandle().tileEntities.size() : 0;
   }

   public int averagePing() {
      List<EntityPlayer> playerList = ((CraftServer)Bukkit.getServer()).getHandle().players;
      return (int)playerList.stream().mapToInt((p) -> {
         return p.ping;
      }).average().orElse(0.0);
   }

   public void setSpawnLimits(World w, int monsters, int animals, int water_animals, int water_ambient, int water_underground_creature, int axolotls, int ambient) {
      w.setMonsterSpawnLimit(monsters);
      w.setAnimalSpawnLimit(animals);
      w.setWaterAnimalSpawnLimit(water_animals);
      w.setWaterAmbientSpawnLimit(water_ambient);
      w.setAmbientSpawnLimit(ambient);
   }

   public double getTps(int time) {
      return ((CraftServer)Bukkit.getServer()).getServer().recentTps[time];
   }

   public void setDistance(int view, int simulation) {
      if (view > 0) {
         Iterator var3 = Bukkit.getServer().getWorlds().iterator();

         while(var3.hasNext()) {
            World w = (World)var3.next();
            ((CraftWorld)w).getHandle().getChunkProvider().setViewDistance(view);
         }
      }

   }

   public void setEntityAi(Entity ent, boolean bl) {
      if (ent instanceof CraftCreature) {
         EntityCreature mob = ((CraftCreature)ent).getHandle();
         if (mob.isNoAI() != bl) {
            return;
         }

         mob.setNoAI(!bl);
         mob.setAggressive(!bl);
         mob.setSilent(!bl);
         mob.collides = !bl;
      }

   }

   @EventHandler
   public void onLoad(ChunkLoadEvent e) {
      SupportManager.EXECUTOR.execute(() -> {
         Entity[] entities = e.getChunk().getEntities();
         CustomAiModule module1 = (CustomAiModule)ModuleManager.getInstance().get(CustomAiModule.class);
         int var6;
         if (module1 != null && module1.canContinue(e.getWorld()) && module1.isLoaded()) {
            CustomAiModule.NMS nmsx = module1.getCustomAi();
            Entity[] var4 = entities;
            int var5 = entities.length;

            for(var6 = 0; var6 < var5; ++var6) {
               Entity entx = var4[var6];
               if (module1.isEnabled(entx)) {
                  nmsx.setCustomAi(entx, false);
               }
            }
         }

         CustomVehiclesModule module2 = (CustomVehiclesModule)ModuleManager.getInstance().get(CustomVehiclesModule.class);
         if (module2 != null && module2.canContinue(e.getWorld()) && module2.isLoaded()) {
            CustomVehiclesModule.NMS nms = module2.getCustomVehicles();
            Entity[] var15 = entities;
            var6 = entities.length;

            for(int var18 = 0; var18 < var6; ++var18) {
               Entity entxx = var15[var18];
               if (module2.isEnabled(entxx)) {
                  nms.setCustomVehicle(entxx);
               }
            }
         }

         SupportManager support = SupportManager.getInstance();
         Set<LivingEntity> creatures = support.getCreatureSet(e.getWorld());
         Set<Item> items = support.getItemSet(e.getWorld());
         Set<Projectile> projectiles = support.getProjectileSet(e.getWorld());
         Entity[] var20 = entities;
         int var9 = entities.length;

         for(int var10 = 0; var10 < var9; ++var10) {
            Entity ent = var20[var10];
            if (ent instanceof LivingEntity) {
               creatures.add((LivingEntity)ent);
            } else if (ent instanceof Item) {
               items.add((Item)ent);
            } else if (ent instanceof Projectile) {
               projectiles.add((Projectile)ent);
            }
         }

      });
   }
}
